; FILE: ddf1.asm




; Ducky direction finder controller for 16C84




;
; 12/07/95 Started work on this file     ...WAR...
;




; Assemble with MPASM /e /l /rDEC /p16C84 ddf1.asm




; RA0/RA1 are used to drive antenna switching.
; RA2/RA3 are used to drive phase indicators.
; RA4 is the audio input.




        include "p16c84.inc"




; Set config fuses to 0x1a
;
; HS oscillator (6 MHz Xtal)
; Watchdog timer disabled
; Power-up timer enabled
; Code protection off




; with a 6 Mhz  Xtal, each cycle is 66.667 usec long
; for a basic 400 Hz tone, the cycle is 2.5 msec.
; This means a half cycle is 1.25 msec, or 1875 CPU cycles.




; our variables
cnt     equ     0x0c
tmp     equ     0x0d




        org   0x000         ; The main line code starts here

; First we do all required initialization

Start

; Set low nybble of port A as output
        bsf     STATUS,RP0      ;Bank 1
        movlw   0xf0
        movwf   TRISA

; Switch back to bank 0
        bcf     STATUS,RP0      ;Bank 0

; Ready for main loop
Top     bsf     PORTA,0         ;Turn right antenna on
        bcf     PORTA,1         ;Turn left antenna off
        bcf     PORTA,2
        btfsc   PORTA,4
        bsf     PORTA,2
        call    haf             ;wait a bit
        nop                     ;Burn 2 cycles for symmetry
        nop
        bsf     PORTA,1         ;Turn left antenna on
        bcf     PORTA,0         ;Turn right antenna off
        bcf     PORTA,3
        btfsc   PORTA,4
        bsf     PORTA,3
        call    haf             ;wait a bit
        goto    Top




; delay for half a cycle of 400 Hz.

; A 400 HZ tone is 2.5 milliseconds
; in duration.  This means each half cycle
; of the tone is 1.25 milliseconds.
; Since I'm using a 6.0 MHz Xtal, this
; is 1875 CPU cycles per tone half cycle.

; Since the routine that calls this to generate
; the tone takes 9 CPU cycles per Tone half
; cycle, we need to burn up 1866 CPU cycles
; in this routine.

; This routine takes 10 + ((K - 1) * 16) cycles
; where K is the argument for the movlw
; instruction.

haf

; we need an extra 4 cycles of padding
; to get the time to come out right
        nop
        nop
        nop
        nop

; now we run the main loop 116 times

        movlw   117
        movwf   cnt

hafl    decfsz  cnt,1
        goto    dowt
        return

dowt    call    dly
        goto    hafl




; Burn up 9 cycles.

dly     movlw   2
        movwf   tmp
dlp     decfsz  tmp,1
        goto    dlp
        return




        end

; Wynn Rostek
; war@palmnet.net
; wynn@pitcairn.ksc.nasa.gov
; wynn.rostek@ksc.nasa.gov
;
; Other Email addresses available if you really need 'em...

